// NimEx1.java

import ch.aplu.jgamegrid.*;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.util.ArrayList;

public class NimEx1 extends GameGrid
  implements GGMouseListener, GGButtonListener
{
  private int nbMatches;
  private int nbTakenMatches;
  private GGBackground bg;
  private GGButton okBtn = new GGButton("sprites/ok.gif", true);
  private GGButton newBtn = new GGButton("sprites/new.gif", true);

  public NimEx1()
  {
    super(56, 9, 12, false);
    bg = getBg();
    addMouseListener(this, GGMouse.lClick);
    addActor(okBtn, new Location(50, 4));
    okBtn.addButtonListener(this);
		addActor(newBtn, new Location(50, 4));
    newBtn.addButtonListener(this);
    init();
    show();
  }

  public void init()
  {
    nbMatches = 15;
    nbTakenMatches = 0;
    bg.clear();
    for (int i = 0; i < 15; i++)
      addActor(new Match(), new Location(2 + 3 * i, 4));
    okBtn.show();
    newBtn.hide();
    refresh();
    setTitle(nbMatches + " matches. Click on 1, 2 or 3 matches to remove, then click 'OK'.");
  }

  public boolean mouseEvent(GGMouse mouse)
  {
    Location loc = toLocationInGrid(mouse.getX(), mouse.getY());
    Actor actor = null;
    for (int y = 2; y < 7; y++)
    {
      actor = getOneActorAt(new Location(loc.x, y), Match.class);
      if (actor != null)
        break;
    }
    if (actor != null)
    {
      if (nbTakenMatches == 3)
        setTitle("Take a maximum of 3. Click 'OK' to continue!");
      else
      {
        actor.removeSelf();
        nbMatches--;
        setTitle(nbMatches + " matches remaining. Click 'OK' to continue.");
        nbTakenMatches++;
        if (nbMatches == 0)
        {
          setTitle("Press 'new Game' to play again.");
          bg.setPaintColor(Color.red);
          bg.setFont(new Font("Arial", Font.BOLD, 90));
          bg.drawText("You lost!", new Point(toPoint(new Location(3, 6))));
					okBtn.hide();
          newBtn.show();
        }
        refresh();
      }
    }
    return false;
  }

  public void computerMove()
  {
    int nbRemovedMatches = nbMatches - nbWin(nbMatches);
    if (nbRemovedMatches == 0)
      nbRemovedMatches = 1; // optimal strategy impossible
    nbMatches = nbMatches - nbRemovedMatches;
    for (int x = 0; x < nbRemovedMatches; x++)
    {
      ArrayList<Actor> matches = getActors(Match.class);
      int k = (int)(Math.random() * matches.size());
      removeActor(matches.get(k)); // take random match
    }
    setTitle(nbMatches + " matches remaining. Your move now.");

    if (nbMatches == 0)
    {
      bg.setPaintColor(Color.red);
      bg.setFont(new Font("Arial", Font.BOLD, 96));
      bg.drawText("You win!", new Point(toPoint(new Location(3, 6))));
      okBtn.hide();
			newBtn.show();
      setTitle("You win, press 'New Game' to play again.");
    }
    refresh();
  }

  // returns next lower winning number
  private int nbWin(int n)
  {
    return ((n - 1) / 4) * 4 + 1;
  }

  public void buttonClicked(GGButton button)
  {
    if (nbMatches == 0)
        init();
      else
      {
        if (nbTakenMatches == 0)
          setTitle("You have to remove at least 1 match!");
        else
        {
          nbTakenMatches = 0;
          computerMove();
        }
      }
  }

	public void buttonPressed(GGButton button)
  {
  }

  public void buttonReleased(GGButton button)
  {
  }

  public static void main(String[] args)
  {
    new NimEx1();
  }
}

// -------------class Match--------------------------
class Match extends Actor
{
  public Match()
  {
    super("sprites/match.gif");
  }
}